#region License
// Copyright (c) 2007 James Newton-King
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
#endregion

using System;
using System.Web;
using NUnit.Framework;
using Utilities;
using Utilities.Testing;
using Utilities.Web;

namespace Utilities.Test.Web
{
  public class MockHttpContextFixture : TestFixtureBase
  {
    [Test]
    public void CreateSecureContext()
    {
      HttpContext context = (new MockHttpContext(true)).Context;
      Assert.IsTrue(context.Request.IsSecureConnection, "The connection is not secure.");
    }

    [Test]
    public void CreateUnsecureContext()
    {
      HttpContext context = (new MockHttpContext(false)).Context;
      Assert.IsFalse(context.Request.IsSecureConnection, "The connection is not secure.");
    }

    [Test]
    public void UseSessionState()
    {
      HttpContext context = (new MockHttpContext(false)).Context;
      Assert.AreEqual(null, context.Session["test"] as string, "The session value is incorrect.");
      context.Session["test"] = "test value";
      Assert.AreEqual("test value", context.Session["test"] as string, "The session value (after setting it) is incorrect.");
    }
  }
}
